"use strict";
/**
 * @module parjs/combinators
 */
/** */
Object.defineProperty(exports, "__esModule", { value: true });
const combinator_1 = require("./combinator");
const parser_1 = require("../parser");
/**
 * Applies the source parser and makes sure its result fulfills `predicate`.
 * @param predicate The condition to check for.
 */
function must(predicate) {
    return combinator_1.defineCombinator(source => {
        return new class Must extends parser_1.ParjserBase {
            constructor() {
                super(...arguments);
                this.type = "must";
                this.expecting = `internal parser ${source.type} yielding a result satisfying condition`;
            }
            _apply(ps) {
                source.apply(ps);
                if (!ps.isOk) {
                    return;
                }
                let result = predicate(ps.value, ps.userState);
                if (result === true)
                    return;
                ps.kind = result.kind || "Soft";
                ps.reason = result.reason || "failed to fulfill a predicate";
            }
        }();
    });
}
exports.must = must;

//# sourceMappingURL=must.js.map