"use strict";
/** @module parjs/combinators */ /** */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const combinator_1 = require("./combinator");
const parser_1 = require("../parser");
const defaults_1 = __importDefault(require("lodash/defaults"));
/**
 * When the source parser is applied, the user state will be switched for a
 * different object. After it has finished, the previous user state will be
 * restored. This effectively isolates the source parser's user state.
 *
 * If the given paramter is a function, it will be called on the pre-existing user
 * state to determine the new user state. If it's a non-fuction object, it will
 * be used as the user state instead.
 *
 * @param innerStateOrCtor The new internal user state or a projection on the
 * existing user state.
 */
function replaceState(innerStateOrCtor) {
    return combinator_1.defineCombinator(source => {
        return new class IsolateState extends parser_1.ParjserBase {
            constructor() {
                super(...arguments);
                this.type = "replaceState";
                this.expecting = source.expecting;
            }
            _apply(ps) {
                let state = ps.userState;
                if (typeof innerStateOrCtor === "function") {
                    ps.userState = defaults_1.default(new parser_1.ParserUserState(), innerStateOrCtor(state));
                }
                else {
                    ps.userState = defaults_1.default(new parser_1.ParserUserState(), innerStateOrCtor);
                }
                source.apply(ps);
                ps.userState = state;
            }
        }();
    });
}
exports.replaceState = replaceState;

//# sourceMappingURL=replace-state.js.map