/** @module parjs/combinators */ /** */
import { Parjser } from "../parjser";
/**
 * Returns a parser that parses any single character. Equivalent
 * to `stringLen(1)`.
 */
export declare function anyChar(): Parjser<string>;
/**
 * Returns a parser that parses a single ASCII inline space.
 */
export declare function space(): Parjser<string>;
/**
 * Returns a parser that parses at least one ASCII inline space.
 */
export declare function spaces1(): Parjser<string>;
/**
 * Returns a parser that parses any character in the string `options`.
 * @param options A string of alternative characters.
 */
export declare function anyCharOf(options: string): Parjser<string>;
/**
 * Returns a parser that parses any character not in the string `exclusions`.
 * @param exclusions A string of characters to exclude.
 */
export declare function noCharOf(exclusions: string): Parjser<string>;
/**
 * Returns a parser that parses a single ASCII letter.
 */
export declare function letter(): Parjser<string>;
/**
 * Returns a parser that parses a single Unicode letter.
 */
export declare function uniLetter(): Parjser<string>;
/**
 * Returns a parser that parses a single digit in base `base`.
 */
export declare function digit(base?: number): Parjser<string>;
/**
 * Returns a parser that parses a single Unicode decimal digit.
 */
export declare function uniDecimal(): Parjser<string>;
/**
 * Returns a parser that parses a single hex digit.
 */
export declare function hex(): Parjser<string>;
/**
 * Returns a parser that parses a single ASCII lowercase letter.
 */
export declare function lower(): Parjser<string>;
/**
 * Returns a parser that parses a single uppercase ASCII letter.
 */
export declare function upper(): Parjser<string>;
/**
 * Returns a parser that parses a single lowercase Unicode letter.
 */
export declare function uniLower(): Parjser<string>;
/**
 * Returns a parser that parses any amount of ASCII whitespace characters
 * and yields the parsed text.
 */
export declare function whitespace(): Parjser<string>;
