"use strict";
/**
 * @module parjs
 */
/** */
Object.defineProperty(exports, "__esModule", { value: true });
const result_1 = require("../result");
const parser_1 = require("../parser");
const char_info_1 = require("char-info");
function innerNewline(unicodeRecognizer) {
    return new class Newline extends parser_1.ParjserBase {
        constructor() {
            super(...arguments);
            this.expecting = "expecting newline";
            this.type = "newline";
        }
        _apply(ps) {
            let { position, input } = ps;
            if (position >= input.length) {
                ps.kind = result_1.ResultKind.SoftFail;
                return;
            }
            let charAt = input.charCodeAt(position);
            if (charAt === char_info_1.AsciiCodes.newline) {
                ps.position++;
                ps.value = "\n";
                ps.kind = result_1.ResultKind.Ok;
                return;
            }
            else if (charAt === char_info_1.AsciiCodes.carriageReturn) {
                position++;
                if (position < input.length && input.charCodeAt(position) === char_info_1.AsciiCodes.newline) {
                    ps.position = position + 1;
                    ps.value = "\r\n";
                    ps.kind = result_1.ResultKind.Ok;
                    return;
                }
                ps.position = position;
                ps.value = "\r";
                ps.kind = result_1.ResultKind.Ok;
                return;
            }
            else if (unicodeRecognizer && unicodeRecognizer(charAt)) {
                ps.position++;
                ps.value = input.charAt(position);
                ps.kind = result_1.ResultKind.Ok;
                return;
            }
            ps.kind = result_1.ResultKind.SoftFail;
        }
    }();
}
/**
 * Parses an ASCII newline, which can be a single character or the sequence
 * `\r\n`. Yields the text that was parsed.
 */
function newline() {
    return innerNewline();
}
exports.newline = newline;
/**
 * Parses a Unicode newline, which includes ASCII newline strings as well as
 * other vertical separators such as PARAGRAPH SEPARATOR.
 */
function uniNewline() {
    return innerNewline(char_info_1.uniIsNewline.code);
}
exports.uniNewline = uniNewline;

//# sourceMappingURL=newline.js.map